import React, { createContext, useContext as useContext$1, useReducer, useMemo, useState, useEffect, useRef, createElement } from 'react';
import process from 'process';

function styleInject(css, ref) {
  if ( ref === void 0 ) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') { return; }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css_248z = ".rsh-input:focus{outline:none}.rsh-search-wrapper{position:relative}.rsh-input-box{background-color:#fff;border:1px solid transparent;border-radius:5px;box-shadow:0 1px 3px 0 rgba(0,0,0,.1),0 1px 2px 0 rgba(0,0,0,.06);min-width:200px}.rsh-input-box-logo{align-items:center;display:flex;height:100%;justify-content:center;margin:0;width:40px}.rsh-input{border:none;border-radius:inherit;font-size:15px;height:100%;margin:0;padding:0;width:100%}.rsh-search-list{background-color:#fff;box-shadow:0 1px 3px 0 rgba(0,0,0,.1),0 1px 2px 0 rgba(0,0,0,.06);box-sizing:border-box;min-width:200px;overflow-x:scroll;overflow:scroll;padding:0 8px;position:absolute;top:40px;width:100%;z-index:12}.rsh-search-list-item{border-radius:13px;list-style:none;margin:10px 0;padding:3px 8px}.rsh-search-list-item__active{background-color:hsla(0,0%,47%,.05)}.rsh-search-list-item-text{margin:0}";
styleInject(css_248z);

function _extends() {
  _extends = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };
  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

var SET_INPUT = 'SET_INPUT';
var START_LOADING = 'START_LOADING';
var END_LOADING = 'END_LOADING';
var SEARCH_DATA = 'SEARCH_DATA';
var DEBOUNCE = 'DEBOUNCE';
var CHARACTER_MATCHING = 'CHARACTER_MATCHING';
var __DEV__ = !process.env.NODE_ENV || process.env.NODE_ENV === 'development';

var reducer = function reducer(state, action) {
  switch (action.type) {
    case SET_INPUT:
      return _extends({}, state, {
        input: action.payload
      });

    case SEARCH_DATA:
      return _extends({}, state, {
        searchData: action.payload
      });

    case START_LOADING:
      return _extends({}, state, {
        isLoading: true
      });

    case END_LOADING:
      return _extends({}, state, {
        isLoading: false
      });

    default:
      return state;
  }
};

var initialState = {
  isLoading: false,
  searchData: undefined,
  input: ''
};
var context = /*#__PURE__*/createContext(null);
var useContext = function useContext() {
  return useContext$1(context);
};
var ContextProvider = function ContextProvider(_ref) {
  var children = _ref.children;

  var _useReducer = useReducer(reducer, initialState),
      state = _useReducer[0],
      dispatch = _useReducer[1];

  var startLoading = function startLoading() {
    dispatch({
      type: START_LOADING
    });
  };

  var endLoading = function endLoading() {
    dispatch({
      type: END_LOADING
    });
  };

  var value = useMemo(function () {
    return {
      state: state,
      dispatch: dispatch,
      startLoading: startLoading,
      endLoading: endLoading
    };
  }, [state]);
  return React.createElement(context.Provider, {
    value: value
  }, children);
};

var isMatch = function isMatch(obj, _char, keys) {
  return keys.some(function (match) {
    return obj[match].toLowerCase().includes(_char);
  });
};
var replaceAll = function replaceAll(obj, regex, keys) {
  var newObj = _extends({}, obj);

  keys.forEach(function (key) {
    return newObj[key] = obj[key].replaceAll(regex, function (match) {
      return "(<mark>" + match + "</mark>)";
    });
  });
  return newObj;
};

/**
 * @param keys - keys to search for from available data
 * @returns {callback} - dispatches action to update search results after character strings
 */

var useCharacterMatching = function useCharacterMatching(keys) {
  var _useContext = useContext(),
      dispatch = _useContext.dispatch;

  return function (input, data) {
    var inputArr = input.toLowerCase().split('');

    if (inputArr.length === 0) {
      dispatch == null ? void 0 : dispatch({
        type: SEARCH_DATA,
        payload: []
      });
      return;
    }

    var regex = new RegExp(inputArr.join('|'), 'gi');

    if (!Array.isArray(data) || !Array.isArray(keys)) {
      if (__DEV__) {
        console.error("Element type is invalid: expected a array but got: " + typeof data + "." + 'This could happen because you have passed wrong data type in data props');
      }

      return;
    }

    if (data.length > 0 && !Object.keys(data[0]).some(function (key) {
      return keys.includes(key);
    })) {
      if (__DEV__) {
        console.error('Keys are invalid: cannot perform matching operation. \n' + 'This could happen for one of the following reasons:\n' + '\t1. keysToSearch does not contain any key for which data object has valid value' + '\t2. You might have passed wrong data types in keysToSearch props\n');
      }

      return;
    }

    var newArr = data == null ? void 0 : data.filter(function (single) {
      return inputArr.every(function (inputChar) {
        return isMatch(single, inputChar, keys);
      });
    }).map(function (item) {
      var newItem = replaceAll(item, regex, keys);
      return _extends({}, item, newItem);
    });
    dispatch == null ? void 0 : dispatch({
      type: SEARCH_DATA,
      payload: newArr
    });
  };
};

var useDebounce = function useDebounce(value, delay) {
  var _useState = useState(value),
      debouncedValue = _useState[0],
      setDebouncedValue = _useState[1];

  useEffect(function () {
    var handler = setTimeout(function () {
      setDebouncedValue(value);
    }, delay);
    return function () {
      clearTimeout(handler);
    };
  }, [value, delay]);
  return debouncedValue;
};

/**
 * Triggers callback function when deps update skipping initial mount
 * @param deps dependency array for useEffect hook
 * @param func callback - triggered on keydown event
 */

var useDidMountEffect = function useDidMountEffect(func, deps) {
  var didMount = useRef(false);
  useEffect(function () {
    if (didMount.current) func();else didMount.current = true;
  }, deps);
};

/**
 * Triggers callback function on keydown while holding cmd
 * @param callback function - triggered on keydown event
 * @param withCmd boolean - with command key pressed
 * @param eventKey string - keydown event key
 * @param deps array - dependencies
 */

var useKeyDown = function useKeyDown(callback, withCmd, eventKey, deps) {
  if (eventKey === void 0) {
    eventKey = 'k';
  }

  if (deps === void 0) {
    deps = [];
  }

  useEffect(function () {
    var keydownCb = function keydownCb(event) {
      var key = event['key'];
      var cmdHeld = withCmd ? event['metaKey'] : true;

      if (cmdHeld && key.toLowerCase() === eventKey) {
        callback == null ? void 0 : callback();
      }
    };

    document.addEventListener('keydown', keydownCb);
    return function () {
      document.removeEventListener('keydown', keydownCb);
    };
  }, deps);
};

var useOffScreen = function useOffScreen(ref) {
  var _useState = useState(false),
      state = _useState[0],
      setState = _useState[1];

  useEffect(function () {
    function handleClickEvent(event) {
      if (ref.current && !ref.current.contains(event.target)) setState(false);
    }

    document.addEventListener('mousedown', handleClickEvent);
    return function () {
      document.removeEventListener('mousedown', handleClickEvent);
    };
  }, [ref]);
  return [state, setState];
};

/**
 * @param keys - keys to search for from available data
 * @returns {callback} - dispatches action to update search results after matching strings
 */

var useStringMatching = function useStringMatching(keys) {
  var _useContext = useContext(),
      dispatch = _useContext.dispatch;

  return function (input, data) {
    if (input.length === 0) {
      dispatch == null ? void 0 : dispatch({
        type: SEARCH_DATA,
        payload: []
      });
      return;
    }

    var regex = new RegExp(input, 'gi');

    if (!Array.isArray(data) || !Array.isArray(keys)) {
      if (__DEV__) {
        console.error("Element type is invalid: expected a array but got: " + typeof data + "." + 'This could happen because you have passed wrong data type in data props');
      }

      return;
    }

    if (data.length > 0 && !Object.keys(data[0]).some(function (key) {
      return keys.includes(key);
    })) {
      if (__DEV__) {
        console.error('Keys are invalid: cannot perform matching operation. \n' + 'This could happen for one of the following reasons:\n' + '\t1. keysToSearch does not contain any key for which data object has valid value' + '\t2. You might have passed wrong data types in keysToSearch props\n');
      }

      return;
    }

    var newArr = data == null ? void 0 : data.filter(function (single) {
      return isMatch(single, input, keys);
    }).map(function (item) {
      var newItem = replaceAll(item, regex, keys);
      return _extends({}, item, newItem);
    });
    dispatch == null ? void 0 : dispatch({
      type: SEARCH_DATA,
      payload: newArr
    });
  };
};

var useThrottle = function useThrottle(value, ms) {
  if (ms === void 0) {
    ms = 200;
  }

  var _useState = useState(value),
      state = _useState[0],
      setState = _useState[1];

  var timeout = useRef();
  var nextValue = useRef(null);
  var hasNextValue = useRef(0);
  var fnRef = useRef(null);
  useEffect(function () {
    if (!timeout.current) {
      setState(value);

      var timeoutCallback = function timeoutCallback() {
        if (hasNextValue.current) {
          hasNextValue.current = false;
          setState(nextValue.current);
          timeout.current = setTimeout(timeoutCallback, ms);
        } else {
          timeout.current = undefined;
        }
      };

      timeout.current = setTimeout(timeoutCallback, ms);
    } else {
      nextValue.current = value;
      hasNextValue.current = true;
    }
  }, [value]);

  var fn = function fn() {
    timeout.current && clearTimeout(timeout.current);
  };

  fnRef.current = fn;
  useEffect(function () {
    return function () {
      return fnRef.current();
    };
  }, []);
  return state;
};

var _excluded = ["as", "children", "classNames"];

var Text = function Text(_ref) {
  var as = _ref.as,
      children = _ref.children,
      classNames = _ref.classNames,
      any = _objectWithoutPropertiesLoose(_ref, _excluded);

  return createElement(as, _extends({
    className: classNames
  }, any), children);
};

var _excluded$1 = ["children"],
    _excluded2 = ["children", "optionIndex"],
    _excluded3 = ["className", "value", "as"];
var InternalContextInitialState = {
  isPopoverExpanded: false,
  listItemActiveIndex: 0,
  updateInternalContext: function updateInternalContext(key, value) {},
  dataLength: 0
};
var InternalContext = /*#__PURE__*/createContext(InternalContextInitialState);
var Wrapper = function Wrapper(_ref) {
  var children = _ref.children;

  var _useState = useState(InternalContextInitialState),
      state = _useState[0],
      setState = _useState[1];

  var updateInternalContext = function updateInternalContext(key, value) {
    setState(function (prev) {
      var _extends2;

      return _extends({}, prev, (_extends2 = {}, _extends2[key] = value, _extends2));
    });
  };

  var listRef = useRef(null);

  var _useOffScreen = useOffScreen(listRef),
      isListVisible = _useOffScreen[0],
      setIsListVisible = _useOffScreen[1];

  useDidMountEffect(function () {
    updateInternalContext('isPopoverExpanded', isListVisible);
  }, [isListVisible]);
  return React.createElement(InternalContext.Provider, {
    value: _extends({}, state, {
      updateInternalContext: updateInternalContext
    })
  }, React.createElement("section", {
    ref: listRef,
    className: "rsh-search-wrapper",

    /**
     * onFocusCapture is triggered when input(child) is focused
     */
    onFocusCapture: function onFocusCapture() {
      return setIsListVisible(true);
    }
  }, children));
};
var PopOverList = function PopOverList(_ref2) {
  var children = _ref2.children,
      any = _objectWithoutPropertiesLoose(_ref2, _excluded$1);

  var listRef = useRef(null);

  var _React$useContext = React.useContext(InternalContext),
      isPopoverExpanded = _React$useContext.isPopoverExpanded,
      listItemActiveIndex = _React$useContext.listItemActiveIndex,
      updateInternalContext = _React$useContext.updateInternalContext,
      dataLength = _React$useContext.dataLength;

  var listItemActiveIndexArrowDown = function listItemActiveIndexArrowDown() {
    if (listItemActiveIndex < dataLength - 1) updateInternalContext('listItemActiveIndex', listItemActiveIndex + 1);
  };

  var listItemActiveIndexArrowUp = function listItemActiveIndexArrowUp() {
    if (listItemActiveIndex > 0) updateInternalContext('listItemActiveIndex', listItemActiveIndex - 1);
  };

  useKeyDown(listItemActiveIndexArrowDown, false, 'arrowdown', [listItemActiveIndex, dataLength]);
  useKeyDown(listItemActiveIndexArrowUp, false, 'arrowup', [listItemActiveIndex]);
  return React.createElement(React.Fragment, null, isPopoverExpanded && React.createElement("ul", Object.assign({
    ref: listRef,
    className: "rsh-search-list"
  }, any), children));
};
var PopOverOption = function PopOverOption(_ref3) {
  var children = _ref3.children,
      optionIndex = _ref3.optionIndex,
      any = _objectWithoutPropertiesLoose(_ref3, _excluded2);

  var _React$useContext2 = React.useContext(InternalContext),
      updateInternalContext = _React$useContext2.updateInternalContext,
      listItemActiveIndex = _React$useContext2.listItemActiveIndex;

  if (__DEV__ && typeof optionIndex !== 'number') throw new ReferenceError("optionIndex type is invalid: expected a number but got: " + typeof optionIndex + ". " + "To avoid the error please provide optionIndex in PopOverOption component" + "\n\t{state.searchData?.map((item, index) => (" + "\n\t\t\t<PopOverOption" + "\n\t\t\t\toptionIndex={index}");

  var handleMouserEnter = function handleMouserEnter() {
    updateInternalContext('listItemActiveIndex', optionIndex);
  };

  return React.createElement("li", Object.assign({
    onMouseEnter: handleMouserEnter,
    className: "rsh-search-list-item " + (listItemActiveIndex === optionIndex && 'rsh-search-list-item__active')
  }, any), children);
};
var PopOverOptionText = function PopOverOptionText(_ref4) {
  var _as;

  var className = _ref4.className,
      value = _ref4.value,
      as = _ref4.as,
      any = _objectWithoutPropertiesLoose(_ref4, _excluded3);

  var words = value == null ? void 0 : value.split(/\(([^)]+)\)/);
  as = (_as = as) != null ? _as : 'h3';
  return React.createElement(Text, Object.assign({
    as: as,
    classNames: className + " rsh-search-list-item-text"
  }, any), words == null ? void 0 : words.map(function (word, index) {
    return word[0] === '<' ? React.createElement("span", {
      key: index,
      dangerouslySetInnerHTML: {
        __html: word
      }
    }) : React.createElement("span", {
      key: index
    }, word);
  }));
};

var Modal = function Modal(_ref) {
  var children = _ref.children,
      isOpen = _ref.isOpen,
      onClose = _ref.onClose;

  var __modal = isOpen != null ? isOpen : true;

  return React.createElement(React.Fragment, null, __modal && React.createElement("div", {
    onClick: onClose != null ? onClose : undefined,
    className: "rsh-modal-dialog",
    style: {
      backgroundColor: 'rgba(220,220,220,0.5)',
      position: 'fixed',
      top: 0,
      right: 0,
      bottom: 0,
      left: 0,
      zIndex: 1000
    }
  }, React.createElement("div", {
    className: "rsh-modal-dialog-box-wrapper",
    style: {
      maxWidth: 400,
      margin: 'auto',
      padding: 5
    }
  }, children)));
};

var _excluded$2 = ["as", "children", "className", "onClick", "onMouseDown", "onFocusCapture"];
var Stack = function Stack(_ref) {
  var as = _ref.as,
      children = _ref.children,
      className = _ref.className,
      onClick = _ref.onClick,
      onFocusCapture = _ref.onFocusCapture,
      args = _objectWithoutPropertiesLoose(_ref, _excluded$2);

  return React.createElement("div", {
    style: _extends({
      display: 'flex',
      flexDirection: as === 'HStack' ? 'row' : 'column',
      alignItems: 'center'
    }, args),
    className: className,
    onClick: onClick,
    onFocusCapture: onFocusCapture
  }, children);
};

var searchIcon = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<!-- Generator: Adobe Illustrator 23.0.1, SVG Export Plug-In . SVG Version: 6.00 Build 0)  -->\n<svg version=\"1.1\" id=\"icon\" xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" x=\"0px\" y=\"0px\"\n\t width=\"16px\" height=\"16px\" viewBox=\"0 0 16 16\" style=\"enable-background:new 0 0 16 16;\" xml:space=\"preserve\">\n<style type=\"text/css\">\n\t.st0{fill:none;}\n</style>\n<title>search</title>\n<path d=\"M15,14.3L10.7,10c1.9-2.3,1.6-5.8-0.7-7.7S4.2,0.7,2.3,3S0.7,8.8,3,10.7c2,1.7,5,1.7,7,0l4.3,4.3L15,14.3z M2,6.5\n\tC2,4,4,2,6.5,2S11,4,11,6.5S9,11,6.5,11S2,9,2,6.5z\"/>\n<rect id=\"_Transparent_Rectangle_\" class=\"st0\" width=\"16\" height=\"16\"/>\n</svg>";

var _excluded$3 = ["keysToSearch", "inputAlgorithm", "matchingAlgorithm", "data", "value", "onChange", "initialValue"];

var Input = function Input(_ref) {
  var keysToSearch = _ref.keysToSearch,
      inputAlgorithm = _ref.inputAlgorithm,
      matchingAlgorithm = _ref.matchingAlgorithm,
      data = _ref.data,
      controlledValue = _ref.value,
      onChange = _ref.onChange,
      initialValue = _ref.initialValue,
      any = _objectWithoutPropertiesLoose(_ref, _excluded$3);

  var _useState = useState(initialValue != null ? initialValue : ''),
      input = _useState[0],
      setInput = _useState[1];

  var _useContext = useContext(),
      dispatch = _useContext.dispatch;

  var __internalContext = React.useContext(InternalContext);

  var inputRef = useRef(null);
  var searchInput = controlledValue != null ? controlledValue : input;
  var searchTerm = inputAlgorithm(searchInput, 500);
  var matchingFn = matchingAlgorithm(keysToSearch);

  if (__DEV__) {
    if (typeof controlledValue !== 'undefined' && !onChange || typeof controlledValue === 'undefined' && onChange) {
      console.warn('SearchBar input value is changing from controlled to uncontrolled, expected a onChange and value prop or none.');
    }
  }

  useEffect(function () {
    if (data) matchingFn(searchInput, data);
    dispatch == null ? void 0 : dispatch({
      type: SET_INPUT,
      payload: searchInput
    });
  }, [searchTerm, data]);
  useEffect(function () {
    if (__internalContext) __internalContext.updateInternalContext('dataLength', data.length);
  }, [data]);

  var focusInput = function focusInput() {
    var _inputRef$current;

    return inputRef == null ? void 0 : (_inputRef$current = inputRef.current) == null ? void 0 : _inputRef$current.focus();
  };

  useKeyDown(focusInput, true);

  var handleOnChange = function handleOnChange(e) {
    return setInput(e.target.value);
  };

  return React.createElement(Stack, {
    as: "HStack",
    onClick: focusInput,
    justifyContent: 'center',
    className: "rsh-input-box",
    padding: 0,
    height: 40,
    // width={'400px'}
    cursor: 'text'
  }, React.createElement("figure", {
    className: "rsh-input-box-logo"
  }, React.createElement("img", {
    src: searchIcon,
    width: "18px"
  })), React.createElement("input", Object.assign({
    value: controlledValue != null ? controlledValue : input,
    onChange: onChange != null ? onChange : handleOnChange,
    placeholder: "search here",
    className: 'rsh-input',
    autoFocus: true,
    ref: inputRef
  }, any)));
};

var _excluded$4 = ["data", "keysToSearch", "inputAlgorithmTimeout", "inputAlgorithm", "matchingAlgorithm", "onChange", "initialValue"];
var inputAlgorithms = {
  DEBOUNCE: useDebounce,
  THROTTLE: useThrottle,
  DEFAULT: function DEFAULT(e) {
    return e;
  }
};
var matchingAlgorithms = {
  CHARACTER_MATCHING: useCharacterMatching,
  STRING_MATCHING: useStringMatching
};
var SearchBar = function SearchBar(props) {
  var _keysToSearch, _inputAlgorithm, _inputAlgorithmTimeou, _matchingAlgorithm;

  var data = props.data,
      keysToSearch = props.keysToSearch,
      inputAlgorithmTimeout = props.inputAlgorithmTimeout,
      _props$inputAlgorithm = props.inputAlgorithm,
      inputAlgorithm = _props$inputAlgorithm === void 0 ? DEBOUNCE : _props$inputAlgorithm,
      matchingAlgorithm = props.matchingAlgorithm,
      onChange = props.onChange,
      initialValue = props.initialValue,
      any = _objectWithoutPropertiesLoose(props, _excluded$4);

  keysToSearch = (_keysToSearch = keysToSearch) != null ? _keysToSearch : Object.keys(data == null ? void 0 : data[0]);
  inputAlgorithm = (_inputAlgorithm = inputAlgorithm) != null ? _inputAlgorithm : DEBOUNCE;
  inputAlgorithmTimeout = (_inputAlgorithmTimeou = inputAlgorithmTimeout) != null ? _inputAlgorithmTimeou : 500;
  matchingAlgorithm = (_matchingAlgorithm = matchingAlgorithm) != null ? _matchingAlgorithm : CHARACTER_MATCHING;
  return React.createElement(Input, Object.assign({
    keysToSearch: keysToSearch,
    duration: inputAlgorithmTimeout,
    inputAlgorithm: inputAlgorithms[inputAlgorithm],
    matchingAlgorithm: matchingAlgorithms[matchingAlgorithm],
    data: data,
    onChange: onChange,
    initialValue: initialValue
  }, any));
};

export { ContextProvider, InternalContext, Modal, PopOverList, PopOverOption, PopOverOptionText, SearchBar, Wrapper, context, useContext };
//# sourceMappingURL=react-search-highlight.esm.js.map
